#include "fil_file.h"

/**
 * These structs were created by Yung-hsiang Lee, Univ. of Southern California
 * All Copyright reserved.
 */
typedef struct {
/* File header */
  char riff[4];
  long filesize;
  char rifftype[4];
} RiffHeader;

typedef struct {
  char chunk_id[4];
  long chunksize;
} Chunk; 

typedef struct {
  short wFormatTag;
  short nChannels;
  long nSamplesPerSec;
  long nAvgBytesPerSec;
  short nBlockAlign;
} WAVEFORMAT;

typedef struct {
  WAVEFORMAT wf;
  short wBitsPerSample;
} PCMWAVEFORMAT;

typedef struct {
  RiffHeader header;
  Chunk ch_format;
  PCMWAVEFORMAT format;
  Chunk ch_data;
  // Data follows here.
} WaveFile;


/**
 * Read the wave data and converts to floating point format.  Assumes uncompressed
 * PCM format, 16 bit, 8000 fs (no header reading).  
 * lf_obj_handle_t points to the wav file object
 * data points to the data pointer
 * len is the number of samples
 */
int fread_wav_data(lf_obj_handle_t ohandle, double** data, int* len) {

  lf_fhandle_t fhandle = 0;
  ffile_t file;
  char *buf;
  size_t buflen;

  /* read buffer */
  ff_open(fhandle, ohandle, &file);
  ff_getbuf(file, &buf, &buflen);
  ff_close(&file);
  
  /* read wave header and data chunk */
  WaveFile* wf = (WaveFile*)malloc(sizeof(WaveFile));
  memcpy((void*)wf, (void*)buf, sizeof(WaveFile));  
  int chunk_len =  wf->ch_data.chunksize;
  char* chunk = (char*)malloc(chunk_len*sizeof(char));

  int nChannels = (int)wf->format.wf.nChannels;
  int nsamples = chunk_len / nsamples;

  /* convert to floating point */
  float* fdata = malloc(sizeof(float)*nsamples);
  for (int s = 0; s < nsamples; s++) {
    fdata[s] = 0.0;
  }

  if (wf->format.wBitsPerSample == (short)16) {
    short* raw_data = (short*)chunk;
    float factor = (1 >> 15)-1;
    int s = 0, n = 0;
    if (nChannels == 1) {
      for (s = 0; s < nsamples; s++) {
	fdata[s] = (float)raw_data[s]/factor;
      }
    }
    if (nChannels == 2) {
      factor *= (float)nChannels;
      for (s = 0; s < nsamples; s++) {	
	fdata[s] += ((float)raw_data[s*2]+(float)raw_data[s*2+1])/factor;
      }
    }  
    else {
      printf("fread_wav_data: Only 1 or 2 channels currently supported.  Requested %d\n", nChannels);
      return -1;

    }
    
  }
  else {
    printf("fread_wav_data: Only 16 bit samples currently supported  Requested %d\n", (int)wf->format.wBitsPerSample);
    return - 1;
  }
  free(chunk);

  /* normalize and convert to double */
  float max = -1.0, min = 1.0;
  for (int s = 0; s < nsamples; s++) {
    if (max < fdata[s]) max = fdata[s];
    else if (min > fdata[s]) min = fdata[s];
  }
  double factor = 2.0/(max - min);
  
  double* ddata = malloc(nsamples*sizeof(double));
  for (int s = 0; s < nsamples; s++) {
    ddata = (double)fdata[s]*factor;
  }
  
  free(fdata);
  
  *data = ddata;
  *len = nsamples;

  return 0;
}

	     
int f_init_readwav(int numarg, char** args, int blob_len,
		   void* blob, void** f_datap) {
  *fdatap = 0;
  return 0;
}

int f_eval_readwav(lf_obj_handle_t ohandle, int numout, 
		   lf_obj_handle_t *ohandles, void *f_datap) {
  
  double* data;
  int nsamples;
  fread_wav_data(ohandle, &data, &nsamples);
  lf_write_attr((lf_fhandle_t)0, ohandle,  
		WAV_DATA, (off_t)nsamples*sizeof(double), (char*)data);
  lf_write_attr((lf_fhandle_t)0, ohandle, WAV_NSAMPLES, sizeof(char*), (char*)(&nsamples));
  
  free(data);  
  
  return 1;
  
}

int f_fini_readwav(void* f_datap) {
  return 0;
}
