function [passed_times, confidences] = test_track(track, classifiers, varargin)
% Finds likely sound locations in wav data according to the classifiers and
% optionally writes those sounds to disk
%
% INPUT:
% track: the wav data
% classifiers(nstages): the classifier for the sound object of interest; 
%                       nstages>1 if classifiers are learned in a cascaded fashion
% varargin: {the output directory, num samples to write} (optional)
%
% OUTPUT: 
% passed_times: center time (in seconds) locations of sound objects; has close (half-frame) detections removed
% confidences(s): the confidence of passed_times(s)


start_time = cputime;

search_params.window_size = classifiers(1).window_length;
search_params.window_step = floor(search_params.window_size/8);

feature_time = 0;
t1 = cputime;

[fft_data, time_locations] = compute_data_representation_fft(track, classifiers(1).fft_params, search_params);

feature_time = cputime - t1;

num_features = length(classifiers(1).features_used);

features_computed = zeros(num_features, 1);

%[features, time_locations, data_locations] = ...
%        extract_global_features_track(track, classifiers(1).fft_params, search_params);        
 
% now features(num_times, num_features)
%features = transpose(features);        

num_windows = fft_data.num_windows;
data_indices = (1:num_windows);

features = zeros(num_features, num_windows);

num_stages = length(classifiers);

passed_indices = 0;
   

for stage = 1:num_stages

    t1 = cputime;
    
    if stage > 1
        features = features(:, passed_indices);
        fft_data.window_i1 = fft_data.window_i1(passed_indices);
        fft_data.window_i2 = fft_data.window_i2(passed_indices);            
    end
    
    for f = 1:num_features
        if classifiers(stage).features_used(f) & (~features_computed(f))
            features(f, :) = compute_feature_track(fft_data, search_params, f);
            features_computed(f) = 1;                                
        end
    end
    
    feature_time = feature_time + cputime - t1;
    %features(:, 1)'
    
    wcs = classifiers(stage).wcs;
    
    confidences = zeros(size(features, 2), 1);    
    num_trees = length(classifiers(stage).wcs);

    for t = 1:num_trees                     
        node_ids = treevalc(wcs(t).var, wcs(t).cut, wcs(t).left_child, wcs(t).right_child, ...
                            wcs(t).catsplit, features);            
        %disp([num2str(length(node_ids)) '  '  num2str(length(confidences))])
        for s = 1:length(node_ids)
            confidences(s) = confidences(s) + wcs(t).node_confidences(node_ids(s));
        end
    end            
    
    % get passed examples
    passed_indices = zeros(length(node_ids), 1);
    num_passed = 0;
    for s = 1:length(node_ids)
        if confidences(s) > classifiers(stage).threshold
            num_passed = num_passed + 1;
            passed_indices(num_passed) = s;
        end
    end
    

    passed_indices = passed_indices(1:num_passed);

    data_indices = data_indices(passed_indices);

    confidences = confidences(passed_indices);
    
end

disp(['feature time: ' num2str(feature_time)])
disp(['total time: ' num2str(cputime-start_time)])

passed_times = time_locations(data_indices);   

fs = classifiers(1).fft_params.fs;
width = time_locations(1)*2*fs;

[passed_times, confidences2] = remove_close_matches(passed_times, confidences, width/fs/2);

X = -sortrows(-[confidences2 passed_times]);

confidences = X(:, 1);
passed_times = X(:, 2);

num_passed = length(passed_times);

% write results to file
if length(varargin)>0
	output_dir = varargin{1};
	
	nsamples = 10;
	if length(varargin)>1
        nsamples = varargin{2};
	end
	
	for i = 1:min(num_passed, nsamples)
        i1 = floor(passed_times(i)*fs-width/2);
        i2 = i1 + width-1;
        clip = track(i1:i2);
        numstr = num2str(1000+i);
        wavwrite(clip, [output_dir '/sample' numstr(2:end) '.wav']);
	end
end


